@@ -29,7 +29,6 @@ c      ****************
        subroutine sopen
        implicit none
 c              read control parameters
-#include  "ZepManager.h"
 #include  "ZsepManager.h"
 #include  "ZepDirec.h"
 #include  "ZepTrackp.h"
@@ -40,13 +39,23 @@ c              read control parameters
 
         character*150 msg
         character*100 cosmosparam
+        integer i, klena, n
 c
         cosmosparam  = ' '
         read(*, *, end=100) EpicsFile, ConfigFile, SepicsFile, cont,
      *   cosmosparam
 
         call sparmr(SepicsFile)
-
+c          check DCInp
+        if(InputA .eq. 'fix') then
+           if(abs( DCInp.x**2 + DCInp.y**2 + DCInp.z**2 - 1.d0)
+     *        .gt. 1.d-3) then
+              call cerrorMsg('sum of DCInp**2 is far from 1', 0)
+           else
+c              normalize so
+              call epadjdir(DCInp)
+           endif
+        endif
 c
 c                   init epics
         if(cosmosparam .ne. ' ') then
@@ -65,16 +74,35 @@ c                   init epics
         endif
 c              incident energy sampling. initializaiton 
         if( PrimaryFile(1:1) .eq. "+") then
+           Inp1ry = 1
            call siniPrim(PrimaryFile)
         else
+c           PrimaryFile may contain: e.g     ../Input/+primary 
+           Inp1ry = 0
+           i = 1
+           n = klena(PrimaryFile)
+           do i = n-1, 1, -1
+              if(PrimaryFile(i:i) .eq.  '/') then
+c                   the last / found
+                 if(PrimaryFile(i:i+1) .eq. '/+') then
+c                         it shouyld be file name with + at head
+                    Inp1ry = 1
+                    call siniPrim(PrimaryFile)
+                 endif
+                 goto 99
+              endif
+           enddo
+        endif
+ 99     continue
+        if(Inp1ry .eq. 0 ) then
            Inpxyz = 0
            Inpdir = 0
            Inperg  = 0
            Inptime = 0
+           Inpdisk = 0
+           Inpuser = 0
            call ciniSPrim(PrimaryFile)
-
         endif
-
 c
 c            inquire form
         call epqfrm(formx)
@@ -85,7 +113,7 @@ c            inquire form
         endif
 
 
-c        call epqcnf(orgw, abcw)
+        call epqcnf(orgw, abcw)
 
 c        call epqworld(nwld, wstruct) 
 c
@@ -122,13 +150,19 @@ c         ---------------- Error -----------------
        end
        subroutine  siniPrim(file)
        implicit none
-#include  "ZepManager.h"
+#include  "ZsepManager.h"
        character(*) file
-       character*30  inpcond
-       integer klena,icon
+       character*50  inpcond
+       integer klena,icon, loc
+
 
        open(Ioprim, file=file(1:klena(file) ))
        read(Ioprim, '(a)')  inpcond
+       if( index(inpcond, "mul") .gt. 0 ) then
+          Inpmul = 1
+       else
+          Inpmul = 0
+       endif
        if( index(inpcond, "xyz") .gt. 0  ) then
           Inpxyz = 1
        else
@@ -158,32 +192,60 @@ c         ---------------- Error -----------------
        else
           Inptime = 0
        endif
+
+       loc = index(inpcond, "user")
+       if( loc .gt. 0) then
+          Inpuser = 1
+       else
+          Inpuser = 0
+       endif
+
+       loc = index(inpcond, "disk")
+       if( loc .gt. 0) then
+          read(inpcond(loc+4:loc+5), *) Inpdisk
+       else
+          Inpdisk = 0
+       endif
+
        call cskipComment(Ioprim, icon)
        if(icon .ne. 0) then
           call cerrorMsg("Primary file has no #----- line", 0)
        endif
 
        end
-       subroutine ssetip2( aTrack, icon)
+       subroutine ssetip3( aTrack, icon)
        implicit none
-#include  "ZepManager.h"
+#include  "ZsepManager.h"
 #include  "ZepTrack.h"
+#include  "Zsparm.h"
 #include  "Zcode.h"
        record /epTrack/ aTrack
-       integer icon, nf, fc
+       integer  icon  ! output. 0  -- go ahead.   1-- call  this again to set multiple 1ries
+c                      -1--> EOF
+       integer  nf, fc
        integer code, subcode, chg 
-       real*8 erg, xin, yin, zin, wx, wy, wz, u
-       character*80 buf
-       character*20 field(11)
+       real*8 erg, xin, yin, zin, wx, wy, wz, u, norm
+       integer maxfn
+       parameter (maxfn = 13)
+       character*120 buf
+       character*20 field(maxfn)
 
+ 10    continue
        buf = ' '
        subcode = 0
-
        read(Ioprim, '(a)', end= 100 )  buf
+       call kgetField(buf, field, maxfn, nf)
 
-       call kgetField(buf, field, 11, nf)
-
-       
+       if(nf .eq. 0 )  then
+          if( Inpmul .eq. 1) then
+             icon = 1
+             return             !****************
+          else
+c            neglect blank line          
+             goto 10
+          endif
+       endif
+          
        fc = 1
        read(field(fc), *) code
        if(Inpsubcode .eq. 1) then
@@ -204,6 +266,7 @@ c         ---------------- Error -----------------
           fc = fc + 1
           read( field(fc), *)   zin
        endif
+
        if( Inpdir .gt. 0 ) then
           fc = fc + 1
           read( field(fc), *)   wx
@@ -211,12 +274,25 @@ c         ---------------- Error -----------------
           read( field(fc), *)   wy
           fc = fc + 1
           read( field(fc), *)   wz
+c              the acuracy of input data may not be enough
+          norm = sqrt( wx**2 + wy**2 + wz**2 )
+          wx = wx/norm
+          wy = wy/norm
+          wz = wz/norm
        endif
+
        if( Inptime .gt. 0 ) then
           fc = fc + 1
           read( field(fc), *)   aTrack.t 
           aTrack.t = aTrack.t*29.98d0    ! cm (t is nsec)
        endif
+
+       if( Inpuser .gt. 0 ) then
+          fc = fc + 1
+          read( field(fc), *)   aTrack.user
+       endif
+
+
 c         check number of field; some redandant field may exist at the last part, so
 c         nf many not be equal to fc. but must be nf >= fc
        if(nf .lt.  fc) then
@@ -249,11 +325,19 @@ c         nf many not be equal to fc. but must be nf >= fc
           aTrack.pos.x = xin
           aTrack.pos.y = yin
           aTrack.pos.z = zin
+c            for safety
+          PosInp.x = xin
+          PosInp.y = yin
+          PosInp.z = zin
        endif
        if(Inpdir .eq. 1) then
           aTrack.w.x = wx
           aTrack.w.y = wy
           aTrack.w.z = wz
+c            for safety also set next
+          DCInp.x = wx
+          DCInp.y = wy
+          DCInp.z = wz
        endif
        aTrack.p.fm.p(4)  =  erg
        if(Inperg .eq. 1) then
@@ -261,12 +345,11 @@ c         nf many not be equal to fc. but must be nf >= fc
        endif
        icon = 0
        return
- 100   icon = 1
+ 100   icon = -1
        end
 
        subroutine  s1set(icon)
        implicit none
-#include  "ZepManager.h"
 #include  "ZsepManager.h"
 #include  "ZepPos.h"
 #include  "ZepDirec.h"
@@ -292,6 +375,7 @@ c                 init 1 event. (clear arrays etc)
          call epi1ev(jcon)
 
 
+
          if(jcon .ne. 0) then
 c                    jcon=0-->user set the incident
 c                    set incident ptcl(s) by sepics
@@ -300,6 +384,9 @@ c                    set incident ptcl(s) by sepics
          else
             icon = 0       ! this is needed 2004. May 2
          endif
+
+
+
          if(icon .eq. 0) then
 c                  check time using stack area data
             call stimec(icon)
@@ -307,6 +394,7 @@ c                  check time using stack area data
          if(icon .eq. 0) then
 c              time and incident are available  for the generation
 c              generate 1 event
+
             call epgen
 c              end of 1 event
             call epe1ev
@@ -347,41 +435,138 @@ c      *****************
        end
       subroutine ssetip( icon )
       implicit none
-#include  "ZepManager.h"
 #include  "ZsepManager.h"
+      integer icon  !  output.  0
+      icon = 0
+
+      if( Inpmul .eq. 1) then
+c            multiple particles are incident; icon = 1--> end of a bunch
+         do while ( icon .eq. 0 )
+            call ssetip2( icon )
+         enddo
+         if(icon .eq. 1) icon = 0
+      else
+         call ssetip2( icon )
+      endif
+      end
+
+      subroutine ssetip2( icon )
+      implicit none
+
+#include  "ZsepManager.h"
+#include  "ZepTrackp.h"
 #include  "ZepTrack.h"
 #include  "Zep3Vec.h"
 #include  "Zswk.h"
 #include  "Zsparm.h"
+#include  "Zglobalc.h"
       
       record /epTrack/ aTrack
       record /epTrack/InciTrack, bTrack
       integer ns, i, j, icon
-      real*8 sume
+      logical ok
+      real*8 sume, theta, phi, r
+      record /epPos/ org, rxyz
+      record /epDirec/ norm
+      character*40 msg
+
+
 
 c             set incident ptcl
 c               fix energy (momentum yet)
-      if(Inperg .eq. 0) then
+      if(Inp1ry .eq. 0 ) then
          call sfixe(aTrack)
          icon = 0
       else
-         call ssetip2(aTrack, icon)
-         if(icon .ne. 0) return ! ************
+         call ssetip3(aTrack, icon)
+         if(icon .ne. 0) return ! ************ EOF or end of 1 bunch of incidents
       endif
 
 c               fix position
-      if(Inpxyz .eq. 0) then
-         call sfixp(aTrack)
-      endif
+      ok = .false.
+
+
+c      do while (.not. ok)
+         if(Inpxyz .eq. 0) then
+            call sfixp(aTrack)
+         endif
 c               fix angle
-      if(Inpdir .eq. 0) then
-         call sfixa(aTrack)
-      endif
+         if(Inpdir .eq. 0) then
+            call sfixa(aTrack)
+         endif
+         if(Inpxyz .eq. 0.) then
+            if(InputP .eq. 'fix') then
+               aTrack.pos=PosInp
+            elseif(InputP .eq. 'usph' .or. InputP .eq. 'g->sph'
+     *           .or. InputP .eq. 'u->sph' ) then
+c                normal vector is the particle position to the center
+               call epqwcoord(org, rxyz)
+               norm.x =-( aTrack.pos.x - org.x )
+               norm.y =-( aTrack.pos.y - org.y )
+               norm.z =-( aTrack.pos.z - org.z )
+               call epadjdir(norm)
+c                  convert direction cos to world coord.
+               call ctransVectZ(norm, DCInp, aTrack.w)
+            elseif(InputP(2:3) .eq. '+z') then
+               aTrack.w.x = DCInp.x
+               aTrack.w.y = DCInp.y
+               aTrack.w.z = DCInp.z
+            elseif(InputP(2:3) .eq. '-z') then
+               aTrack.w.x = DCInp.x
+               aTrack.w.y = DCInp.y
+               aTrack.w.z = -DCInp.z
+            elseif(InputP(2:3) .eq. '+y') then              
+               aTrack.w.x = DCInp.x
+               aTrack.w.y = DCInp.z
+               aTrack.w.z = DCInp.y
+            elseif(InputP(2:3) .eq. '-y') then              
+               aTrack.w.x = DCInp.x
+               aTrack.w.y = -DCInp.z
+               aTrack.w.z = DCInp.y
+            elseif(InputP(2:3) .eq. '+x') then              
+               aTrack.w.x = DCInp.z
+               aTrack.w.y = DCInp.x
+               aTrack.w.z = DCInp.y
+            elseif(InputP(2:3) .eq. '-x') then              
+               aTrack.w.x = -DCInp.z
+               aTrack.w.y = DCInp.x
+               aTrack.w.z = DCInp.y
+            elseif(InputP .eq. 'g->sph2'
+     *        .or. InputP .eq. 'u->sph2' ) then
+c
+c              DCInp gives teta and phi .  opening angle is 90 deg
+               call epqwcoord(org, rxyz)
+                            ! rxyz.x= should be the radius of the sphere
+               r = rxyz.x -EpsLeng
+               theta = acos(-DCInp.z)*Todeg
+               phi = atan2(-DCInp.y, -DCInp.x)*Todeg
+               if(InputP .eq. 'g->sph2') then
+                  call epgonSphere(1, Hwhm, r, 
+     *              theta, phi, 90.0d0, aTrack.pos)
+               else
+                  call epgonSphere(1, -1.0d0, r, 
+     *              theta, phi, 90.0d0, aTrack.pos)
+               endif
+               aTrack.pos.x = aTrack.pos.x + org.x
+               aTrack.pos.y = aTrack.pos.y + org.y
+               aTrack.pos.z = aTrack.pos.z + org.z
+               aTrack.w = DCInp
+            else
+               write(msg,*)' InputP=',InputP, ' and ',
+     *             InputA,' incompatible'
+               call cerrorMsg(msg, 0)
+            endif
+         endif
+c      enddo
 c               fix others
 
       call sfixo(aTrack)
 c               set them
-      call epputTrack(aTrack)
+      if(Inpdisk .eq. 0) then
+         call epputTrack(aTrack)
+      else
+         call epputTrk2(Inpdisk, aTrack)
+      endif
       InciTrack = aTrack
       return
 c      *****************
@@ -438,8 +623,8 @@ c             get  current random no.
        end
       subroutine sfixe(aTrack)
        implicit none
-#include  "ZepManager.h"
-#include  "Zglobalc.h"
+#include  "ZsepManager.h"
+
 #include  "ZepTrack.h"
 #include  "ZepTrackp.h"
 #include  "Zep3Vec.h"
@@ -459,7 +644,7 @@ c             get  current random no.
        real*8 u, r,   dx, dy, cs, sn
        real*8 st, pw
        record /epPos/ org, rxyz
-
+       logical ok
        character*70 msg
 
 
@@ -477,8 +662,8 @@ c          sample incident x,y,z position;  world coord.
 #else
           aTrack.pos = PosInp
 #endif
-       elseif(InputP(1:2) .eq. 'u-' .or.
-     *        InputP(1:2) .eq. 'u+' ) then
+       elseif( (InputP(1:2) .eq. 'u-' .and. InputP(1:3) .ne. 'u->')
+     *       .or.   InputP(1:2) .eq. 'u+' ) then
 c            uniform in Xrange, Yrange,  Zrange        
           call rndc(u)
           aTrack.pos.x
@@ -494,7 +679,6 @@ c            uniform in Xrange, Yrange,  Zrange
      *          + real(Zrange)
 
        elseif(InputP .eq. 'usph') then
-c                 InputP = 'usph'
           call epqwcoord(org, rxyz)
                          ! rxyz.x= should be the radius of the sphere
           r = rxyz.x -EpsLeng
@@ -504,11 +688,15 @@ c                                  teta       phi      o.a
           aTrack.pos.x = aTrack.pos.x + org.x
           aTrack.pos.y = aTrack.pos.y + org.y
           aTrack.pos.z = aTrack.pos.z + org.z
-       elseif(InputP .eq. 'gsph') then
+c            angle is later fixed.
+       elseif(InputP .eq. 'g->sph') then
 c
 c           PosInp.x = teta, PosInp.y = phi, PosInp.z = opening angl
-c           ProfR should be  R * sin(O.A), but not referred.
-c
+c           Instead of ProfR,  R * sin(O.A) is used.
+c     
+          if(PosInp.z .gt. 90.) then
+             call cerrorMsg("Zinp > 90 for InputP=g->sph", 0)
+          endif
           call epqwcoord(org, rxyz)
                             ! rxyz.x= should be the radius of the sphere
           r = rxyz.x -EpsLeng
@@ -519,6 +707,30 @@ c
           aTrack.pos.x = aTrack.pos.x + org.x
           aTrack.pos.y = aTrack.pos.y + org.y
           aTrack.pos.z = aTrack.pos.z + org.z
+c            angle is later fixed.
+       elseif(InputP .eq. 'g->sph2') then
+c               this case you  InpDir!=0 or DCInp must be given
+c               wait until angle is given                   
+       elseif(InputP .eq. 'u->sph') then
+          if(PosInp.z .gt. 90.) then
+             call cerrorMsg("Zinp > 90 for InputP=u->sph", 0)
+          endif
+          call epqwcoord(org, rxyz)
+                            ! rxyz.x= should be the radius of the sphere
+          r = rxyz.x -EpsLeng
+c            For  negative HWHM, uniform distribution is employed
+          call epgonSphere(1, -1.0d0, r, PosInp.x, PosInp.y, PosInp.z,
+     *       aTrack.pos)
+
+          aTrack.pos.x = aTrack.pos.x + org.x
+          aTrack.pos.y = aTrack.pos.y + org.y
+          aTrack.pos.z = aTrack.pos.z + org.z
+c            angle is later fixed.
+c  
+       elseif( InputP .eq. 'u->sph2') then
+c               this case  InpDir!=0 or DCInp must be given
+c               wait until angle is given                   
+c  
        elseif(InputP(1:1) .eq. 'g') then
 c                  gaussian density beam
           call epGaussb(Hwhm, ProfR, dx, dy)
@@ -537,27 +749,20 @@ c                 around Xinp, Yinp
              aTrack.pos.z = PosInp.z + dy
           else
              write(msg,*) ' InputP=',InputP, ' invalid'
-             call cerrorMsg(msg,  0)
+             call cerrorMsg(msg,  1)
+             call cerrorMsg("see sepicsfile in FirstKiss",0)
           endif
        else
           write(msg,*) ' InputP=',InputP, ' invalid'
-          call cerrorMsg(msg,  0)
+          call cerrorMsg(msg,  1)
+          call cerrorMsg("see sepicsfile in FirstKiss",0)
        endif
        return
 c       ****************************************
         entry sfixa(aTrack)
-c          
-        if(InputA .eq. 'fix') then
-           aTrack.w.x = DCInp.x
-           aTrack.w.y = DCInp.y
-           aTrack.w.z = DCInp.z
-           if(abs( DCInp.x**2 + DCInp.y**2 + DCInp.z**2 - 1.d0)
-     *        .gt. 1.d-3) then
-              call cerrorMsg('sum of DCInp**2 is far from 1', 0)
-           else
-c              normalize so
-              call epadjdir(aTrack.w)
-           endif
+c          DCInp is fixed. aTrack.w is not yet  given
+        if(InputA .eq. 'fix' ) then
+c          nothing to do; DCInp is used.
         else
            if(InputA .eq. 'is') then
               call rndc(u)
@@ -568,12 +773,12 @@ c              normalize so
               call rndc(u)
               if(pw .ne. -1.d0) then
                  DCInp.z=
-     *          ( (IMAG_P(CosNormal)**(pw+1.d0) - 
-     *                  real(CosNormal)**(pw+1.d0))* u
-     *           + real(CosNormal)**(pw+1.d0))**(1.d0/(pw+1.d0))
+     *             ( (IMAG_P(CosNormal)**(pw+1.d0) - 
+     *             real(CosNormal)**(pw+1.d0))* u
+     *             + real(CosNormal)**(pw+1.d0))**(1.d0/(pw+1.d0))
               else
                  DCInp.z =(IMAG_P(CosNormal)/real(CosNormal))**u *
-     *                  real(CosNormal)
+     *             real(CosNormal)
               endif
            else
               write(msg,*) ' InputA=',InputA, ' not supported'
@@ -583,61 +788,12 @@ c              normalize so
            st= sqrt( 1.d0 - DCInp.z**2)
            DCInp.x = st * cs
            DCInp.y = st * sn
-
-           if(InputP .eq. 'fix') then
-              aTrack.w.x = DCInp.x
-              aTrack.w.y = DCInp.y
-              aTrack.w.z = DCInp.z
-           elseif(InputP .eq. 'usph' .or.
-     *            InputP .eq. 'gsph' ) then
-              if( DCInp.x*(aTrack.pos.x -org.x)+ 
-     *            DCInp.y*(aTrack.pos.y -org.y)+ 
-     *            DCInp.z*(aTrack.pos.z -org.z) .lt. 0.) then
-                 aTrack.w.x = DCInp.x
-                 aTrack.w.y = DCInp.y
-                 aTrack.w.z = DCInp.z
-              else
-                 aTrack.w.x = -DCInp.x
-                 aTrack.w.y = -DCInp.y
-                 aTrack.w.z = -DCInp.z
-              endif                 
-
-c              direct the angle around the normal vector
-           elseif(InputP(2:3) .eq. '+z') then
-              aTrack.w.x = DCInp.x
-              aTrack.w.y = DCInp.y
-              aTrack.w.z = DCInp.z
-           elseif(InputP(2:3) .eq. '-z') then
-              aTrack.w.x = DCInp.x
-              aTrack.w.y = DCInp.y
-              aTrack.w.z = -DCInp.z
-           elseif(InputP(2:3) .eq. '+y') then              
-              aTrack.w.x = DCInp.x
-              aTrack.w.y = DCInp.z
-              aTrack.w.z = DCInp.y
-           elseif(InputP(2:3) .eq. '-y') then              
-              aTrack.w.x = DCInp.x
-              aTrack.w.y = -DCInp.z
-              aTrack.w.z = DCInp.y
-           elseif(InputP(2:3) .eq. '+x') then              
-              aTrack.w.x = DCInp.z
-              aTrack.w.y = DCInp.x
-              aTrack.w.z = DCInp.y
-           elseif(InputP(2:3) .eq. '-x') then              
-              aTrack.w.x = -DCInp.z
-              aTrack.w.y = DCInp.x
-              aTrack.w.z = DCInp.y
-           else
-              write(msg,*)' InputP=',InputP, ' and ',
-     *             InputA,' incompatible'
-              call cerrorMsg(msg, 0)
-           endif
         endif
         return
 c        ***************************
         entry sfixo(aTrack)
 c              fix time of incidence
-        if( Inptime .eq. 0) then
+        if( Inptime .eq. 0 ) then
            aTrack.t = 0.d0
         endif
 c           fix momentum 
